'(
Version 2.4
Briefly:
R-encoder - tuning/Volume/selection from memory presets
BT1 - shortly - switches Volume/Tuning,  hold - starts scanning- (stop by anything)
BT2 - shortly - Stereo/Mono,             hold - BassBoost ON/Off
BT3 - shortly - Display ON/OFF,          hold - starts scanning+ (stop by anything)
Encoder switch - shortly - program from presets, hold - saves to presets
BT3 hold when power-On - delete all presets
BT1 hold when power-On - calibration LOW BAT (BT2 confirm and save)

All the variables which begin with letter E are Eeprom variables
')


$regfile = "m328pbdef.dat"                                  ' Atmega328PB
$crystal = 8000000                                          ' internal RC oscillator 8MHz
$hwstack = 64                                               ' default use 32 for the hardware stack
$swstack = 32                                               ' default use 10 for the SW stack
$framesize = 64                                             ' default use 40 for the frame space
$lib "I2C_TWI.lbx"                                          ' With this library -  hardware I2C on chip (port_C.4 and port_C.5)
$lib "glcdSSD1306-I2C_V2.lib"                               ' Library for I2C Oled display (SSD1306, alternatively SSH1106)

Const Updateeprom = 0                                       ' Prevents too many writing into Eeprom

Wait 2                                                      'Wait 2 seconds until display resets
'******************************************************************************
'**** Config of I2C port, we use HW I2C on chip with library I2C_TWI.lbx ******
'******************************************************************************
Set Portc.5 : Set Portc.4                                   'Internal pull-up resistors (On the PCB are not pull-up resistors)
Config Scl = Portc.5                                        'I2C clock SCL
Config Sda = Portc.4                                        'I2C data SDA
Config Twi = 400000                                         'I2C speed
I2cinit                                                     'I2C necessary initialization

'********* Config Interrupt Ovf1 Timer1 ****************************************
Enable Interrupts
Config Timer1 = Timer , Prescale = 1024
On Ovf1 Uloz                                                ' (Czech word Uloz means Save)
Stop Timer1
'********* Config Interrupt Ovf3 Timer3 ****************************************
Config Timer3 = Timer , Prescale = 1024
On Ovf3 Read_rds
Stop Timer3
'********* Config ADC for measuring Voltage ************************************
Config Adc = Single , Prescaler = Auto , Reference = Avcc
Stop Adc
'*******************************************************************************
'*** Declaration variables ** Variables with E are in Eeprom *******************
'*******************************************************************************
Dim Kmitocet As Word                                        'input frequency (Kmitocet = Frequency)
Dim E_kmitocet As Eram Word                                 'Frequency saved in Eeprom
Dim Dato0_byte As Byte                                      'Auxiliary variable byte for calculations
Dim Dato1_byte As Byte                                      'Auxiliary variable byte for calculations
Dim Pll As Single                                           'Auxiliary variable single for calculations
Dim Kmitocet_str As String * 5                              'String for display frequency
Dim Smer As Byte                                            'Direction of scanning according to the button (Smer = Direction)
Dim Volume As Byte                                          'Byte for volume
Dim Zobraz_volume As Byte                                   'Auxiliary byte for volume (Zobraz_volume = Show_volume)
Dim E_volume As Eram Byte                                   'Volume byte saved in Eeprom
Dim Audio As Byte                                           'byte for setting stereo, bass/boost, etc...
Dim E_audio As Eram Byte                                    'byte for setting stereo, bass, etc... saved in eeprom
Dim Svetlo As Word : Svetlo = &HAF                          'variable for turn Off display, Tma = &HAE (Tma = Darkness, Svetlo = Light)
Dim Rds(12) As Byte                                         'variable for reading RDS register

Dim A As Byte                                               'auxiliary variable for manipulations with RDS data
Dim B As Byte                                               'auxiliary variable for manipulations with RDS data
Dim Rds_text As String * 8                                  'RDS string for display

Dim S As Byte                                               'auxiliary variable for manipulations with RDS data
Dim T As Byte                                               'auxiliary variable for manipulations with RDS data

Dim E_pamet(20) As Eram Word                                'Array in Eeprom for presets (Pamet = Memory)
Dim Pamet_jmeno As String * 8                               'String of name for preset (Pamet_jmeno = Memory_name)
Dim E_pamet_jmeno(20) As Eram String * 8                    'Array in Eeprom for saved names of stations
Dim Cislo_pameti As Byte                                    'Number of preset   (Cislo_pameti = number_of_memory)
Dim E_posledni_pamet As Eram Byte                           'Byte in Eeprom - the last preset memory
Dim E_nejvyssi_pamet As Eram Byte                           'Byte in Eeprom - the highest preset memory
Dim Nejvyssi_pamet As Byte                                  'Byte keeps the highest preset memory
Dim Pam As Byte : Dim E_pam As Eram Byte                    'Keeps the state of radio, when turned Off (whether in tuning(0) or presets(1) mode)
                                                            'Could be Bit, but in Eeprom cant't exist variable Bit.
Pam = E_pam                                                 'Read the state from Eeprom (after turn radio On)
If Pam > 1 Then                                             'Can be 0 or 1
   Pam = 0                                                  'When higher than 1 then sets 0 (= manual tuning mode)
   E_pam = 0
End If
Dim Volty As Word                                           'Variable Volts for measuring battery
Dim V_limit As Word                                         'Limit when it announces Low bat
Dim E_v_limit As Eram Word                                  'Limit Low_bat saved in Eeprom
V_limit = E_v_limit                                         'Read the Volt limit, when turn radio On
If V_limit > 1023 Then V_limit = 139                        'When "out of range", then sets 139 (about 4 Volts)

Dim Vol_mem As Bit : Vol_mem = 0                            'Auxiliary bit for setting volume in presets (memory) mode

'*******************************************************************************
'***** Aliases of Buttons ******************************************************
'*******************************************************************************
Bt1 Alias Pinc.0 : Config Bt1 = Input : Set Bt1             'tlacitko 1    (button 1)
Bt2 Alias Pinc.1 : Config Bt2 = Input : Set Bt2             'tlacitko 2    (button 2)
Bt3 Alias Pinc.2 : Config Bt3 = Input : Set Bt3             'tlacitko 3    (button 3)
Re_a Alias Pind.0 : Config Re_a = Input : Set Re_a          'encoder switch A
Re_b Alias Pind.1 : Config Re_b = Input : Set Re_b          'encoder switch B
Re_sw Alias Pind.2 : Config Re_sw = Input : Set Re_sw       'encoder switch in axis

'*******************************************************************************
'**** Declarations of special functions for SSH1106 chip of Oled Display *******
'*******************************************************************************
Declare Sub Initssh1106                                     'code of this function is included from external file
Declare Sub Xmbyte(byreg R16 As Byte)                       '   -//-
Declare Sub Clscrn                                          'Function for clear the display (necessary for some displays)

'*******************************************************************************
'**** Configuration of parameters of Oled I2C display **************************
'*******************************************************************************
Dim _contrast As Byte                                       ' variable for brightness [0-255]
Config Graphlcd = Custom , Cols = 128 , Rows = 64 , Lcdname = "SSD1306"       'definition of display
Initssh1106                                                 'initialization of display
Waitms 100                                                  'wait, until reset of display
Cls                                                         'clear the display
Clscrn                                                      'clear the display (some need this special function)
_contrast = 0 : Glcdcmd &H81                                'set minimal brightness of the display

'*******************************************************************************
'**** Configuration of Timer2 as PWM for setting the volume ********************
'*******************************************************************************
Config Portd.3 = Output                                     'pin D.3 as output
Config Timer2 = Pwm , Prescale = 1 , Compare B Pwm = Clear_up       'timer_2 as PWM
Enable Timer2                                               'enable timer_2

'*******************************************************************************
'**** Debounce = "delay" for buttons *******************************************
'*******************************************************************************
Config Debounce = 2                                         'debounce 2 ms

'*******************************************************************************
'***** Declarations of functions ***********************************************
'** It could be done also using GoSub, but this is clearer for me. *************
'*******************************************************************************
Declare Sub Ladeni()                                        'Ladeni = tuning
Declare Sub Nastav()                                        'Nastav = Set  or Adjust
Declare Sub Uloz()                                          'Uloz = save
Declare Sub Nastav_volume()                                 'Nastav_volume = set (adjust) volume
Declare Sub _volume()
Declare Sub Scan()
Declare Sub Stereo()
Declare Sub Bass()
Declare Sub Read_rds()
Declare Sub Bt1_vidle()                                     'Bt1_vidle = Bt1_fork (identify short press or hold button_1)
Declare Sub Bt2_vidle()                                     'Bt2_vidle = Bt2_fork
Declare Sub Bt3_vidle()                                     'Bt3_vidle = Bt3_fork
Declare Sub Re_sw_vidle()                                   'Enoder axis switch fork (identify short press or hold axis switch)
Declare Sub Zhasni()                                        'Zhasni = Turn off the light
Declare Sub Nastav_pamet()                                  'Nastav_pamet  = set the memory
Declare Sub Nastav_pamet_b()                                '    -- // --       B
Declare Sub Uloz_pamet()                                    'Uloz_pamet = save the memory
'Declare Sub Smaz_jmeno()                                   'Not used so far
Declare Sub Pamet_vyber()                                   'Pamet_vyber = Memory choose
Declare Sub Baterie()                                       'Battery
Declare Sub Vyber_pamet()                                   'Vyber_pamet = Choose the memory

'*******************************************************************************
'***** Show picture (some advertisement) not used ******************************
'*******************************************************************************
'(
Clscrn
Showpic 0 , 0 , Logo
Waitms 2000
_contrast = 0 : Glcdcmd &H81
Cls
clscrn
')

'*******************************************************************************
'***** Read initial values saved in Eeprom *************************************
'*******************************************************************************
If Bt3 = 0 Then                                             'When you hold Bt3 while power-on
   E_posledni_pamet = 0                                     'Set the last_memory in Eeprom to 0
   E_nejvyssi_pamet = 0                                     'Set the highest_memory in Eeprom to 0
   Pam = 0 : E_pam = 0                                      'Set the mode "manual tuning"
   Lcdat 1 , 12 , "ERASED"                                  'Display "ERASED"
   Wait 3                                                   'Wait 3 seconds
   Lcdat 1 , 1 , "          "
   Bitwait Bt3 , Set                                        'Wait for release of Bt3
End If

If Bt1 = 0 Then                                             'When you hold Bt1 while power-on
   Lcdat 1 , 24 , "SET LOW"
   Lcdat 3 , 1 , "  BATTERY"                                'Display "Set low battery"
   Bitwait Bt1 , Set                                        'Wait until you release Bt1
   Waitms 200                                               'Wait 200ms more
   Lcdat 7 , 18 , "Push BT2"                                'Display "Push Bt2"
   Start Adc
   Do                                                       'Measure voltage
      Volty = Getadc(3)
      If Bt2 = 0 Then                                       'When you push the Bt2, then...
         V_limit = Volty                                    '...set and...
         E_v_limit = Volty                                  '... save the measured value.
         Stop Adc
         Clscrn                                             'Clear display
         Lcdat 1 , 13 , "SET"                               'And Write "SET"
         Wait 3                                             'Wait 3 seconds
         Clscrn                                             'Clear display
         Exit Do
      End If
   Loop
End If

Volume = E_volume                                           'Read the volume from Eeprom
If Volume > 100 Or Volume < 20 Then Volume = 40             'First turn-on after upload code is 255
Pwm2b = Volume                                              'Send the value to PWM output

Audio = E_audio                                             'Read audio parameters from Eeprom (Stereo, Bass-boost)
If Audio > &B111_10010 Then                                 'First turn-on after upload code is &B1111_1111
   Audio = &B1101_0010 : E_audio = Audio                    'Set proper value and save it to Eeprom
End If

Kmitocet = E_kmitocet                                       'Read the frequency from Eeprom
If Kmitocet < 870 Or Kmitocet > 1080 Then Kmitocet = 897    'If frequency out of range, set my prefered radio station
                                                            'You can change the 897 to your prefered frequency (Kmitocet = Frequency)
Setfont _font12x16                                          'Set display font to 12x16 pixels
Lcdat 5 , 89 , "MHz"                                        'Display "MHz"
Nastav                                                      'Jump to the function of calculations and sending frequency into RDA5807


'*******************************************************************************
'*******************************************************************************
'***** Main loop, waiting for Buttons ******************************************
'*******************************************************************************
'*******************************************************************************
If Pam = 1 Then Nastav_pamet                                'when Pam=1 (read from Eeprom) jump into presets memory mode
                                                            'If Pam=0 then stay in tuning mode
Lcdat 1 , 1 , "T:"                                          'Display "T" as indication of Tuning mode
Lcdat 1 , 33 , "FM RADIO"                                   'If not data from RDS, display fixed text "FM RADIO"
Enable Ovf3                                                 'Enable interruption of Timer_3 (periodically reads RDS data)
Bitwait Re_sw , Set                                         'Wait for release of encoder axis switch (when returning from preset memory mode after turn-on)
Waitms 200                                                  'Wait more 200ms
Start Timer3                                                'Timer3 causes periodically Ovf3 (reads and displays RDS info, see config Ovf3)

Main_loop:
Do
   Vol_mem = 0                                              'Flag Vol_mem is 0 in tuning mode
   Debounce Re_a , 0 , Ladeni , Sub                         'Ladeni = tuning (Re_a = Rotary encoder A switch)
   Debounce Bt1 , 0 , Bt1_vidle , Sub                       'Bt1_vidle =  Bt1_fork (short/long press)
   Debounce Bt2 , 0 , Bt2_vidle , Sub                       'Bt2_vidle = Bt2_fork       -- // --
   Debounce Bt3 , 0 , Bt3_vidle , Sub                       'Bt3_vidle = Bt3_fork       -- // --
   Debounce Re_sw , 0 , Re_sw_vidle , Sub                   'Re_sw_vidle = Re_sw_fork   -- // --    (Re_sw = rotary encoder switch)
Loop

End
'*******************************************************************************
'*******************************************************************************
'*******************************************************************************

'*******************************************************************************
'***** Re_sw vidle - distinguish short pres from holding the button ************
'*******************************************************************************
Sub Re_sw_vidle()
   Timer3 = 50000                                           'I don't remember why I set this, sorry.
   Timer1 = 0                                               'Timer1 is used for measuring the time of holding button
   Start Timer1
  Do                                                        'start loop
      Waitms 25
      If Re_sw = 1 Then                                     'When you release switch soon
         Stop Timer1                                        'stop timer
         Nastav_pamet                                       'short press - nastav_pamet (go to presets memory mode)
         Return
      Elseif Timer1 > 7000 Then                             'If time is longer than 7000 periods then
         Stop Timer1                                        'stop timer
         Uloz_pamet                                         'hold switch - save station into memory (go to function Save_memory)
         Return
      End If
  Loop
End Sub


'*******************************************************************************
'***** Bt1, Bt2, Bt3 vidle - distinguish short pres from holding the button ****
'*******************************************************************************
Sub Bt1_vidle()                                             ' Bt1 fork
   Timer3 = 50000                                           ' I don't remember why I set this, sorry.
   Timer1 = 0                                               ' Timer1 is used for measuring the time of holding button
   Start Timer1
  Do                                                        'start loop
      Waitms 25
      If Bt1 = 1 Then                                       'When you release the switch soon...
         Stop Timer1                                        '... stop timer...
         Nastav_volume                                      '...and go to Set_volume
         Return                                             '...return to mail loop.
      Elseif Timer1 > 7000 Then                             'If time is longer than 7000 periods then...
         Stop Timer1                                        '...stop timer...
         Smer = 1                                           '...set the dirrection of scanning... (smer = dirrection)
         Scan                                               '...and jump to the scanning routine.
         Return                                             'Return to the main loop.
      End If
  Loop
End Sub

Sub Bt2_vidle()                                             'The same for Bt2
   Timer3 = 50000
   Timer1 = 0
   Start Timer1
  Do
      Waitms 25
      If Bt2 = 1 Then                                       'short press
         Stop Timer1
         Stereo                                             '...jump to setting Mono/Stereo
         Return
      Elseif Timer1 > 7000 Then                             'longer hold
         Stop Timer1
         Bass                                               '...jump to Bass-boost On/Of
         Return
      End If
  Loop
End Sub

Sub Bt3_vidle()                                             'The same for Bt3
   Timer3 = 50000
   Timer1 = 0
   Start Timer1
  Do
      Waitms 25
      If Bt3 = 1 Then                                       'short press
         Stop Timer1
         Zhasni                                             '... jump to routine Display On/Off (Zhasni = apaga la luz)
         Return
      Elseif Timer1 > 7000 Then                             'longer hold
         Stop Timer1
         Smer = 2                                           'set the dirrection for scanning   (smer = dirrection)
         Scan                                               '...and jump to the scanning routine
         Return
      End If
  Loop
End Sub

'*******************************************************************************
'***** turn Off/On the display *************************************************
'*******************************************************************************
Sub Zhasni()                                                'Zhasni = turn of the light (apaga la luz)
    Timer3 = 50000
      If Svetlo = &HAF Then                                 'swap &HAF and &HAE
         Svetlo = &HAE                                      ' (svetlo =  light)
      Else
         Svetlo = &HAF
      End If
    Glcdcmd Svetlo                                          'and here apply the value (&HAF or &HAE)
    Bitwait Bt3 , Set                                       'wait for release the button
End Sub

'*******************************************************************************
'***** Scanning band ***********************************************************
'*******************************************************************************
Sub Scan()
   Timer3 = 50000
   Stop Timer3                                              'stop timer3 stops the RDS
   Rds_text = ""                                            'set empty RDS text
   Lcdat 1 , 33 , "        "                                'clear a line on display
   If Smer = 2 Then                                         ' Smer =  dirrection (2 means Up)
      Kmitocet = Kmitocet + 1                               ' change frequency up by 1 step
   Else                                                     ' when Smer is not 2
      Kmitocet = Kmitocet - 1                               ' dirrection down
   End If
   Nastav                                                   'jump to calculation set and display the frequency
   Read_rds                                                 'jump to the reading RDS register
   Bitwait Bt3 , Set                                        'wait for release of the button
   Waitms 200
   Timer1 = 0                                               'Here timer1 is used for speed of the scanning
   Start Timer1
   Do

      If Timer1 > 10000 Then                                'one step for each cca 1.2 sec
         If Smer = 2 Then                                   'dirrection up
            Kmitocet = Kmitocet + 1                         ' (kmitocet = frequency)
            If Kmitocet = 1080 Then Kmitocet = 870          'at the top end of the band start from bottom
         Else
            Kmitocet = Kmitocet - 1                         'dirrection down
            If Kmitocet = 870 Then Kmitocet = 1079          'at the bottom of the band start from top
         End If
         Nastav                                             'jump to calculations, set and display the frequency
         Waitms 200
         Read_rds                                           'jump to the reading RDS register
         Timer1 = 0                                         'set timer1 to 0 for the next step
      End If
      If Re_a = 0 Or Bt1 = 0 Or Bt2 = 0 Or Bt3 = 0 Then     'scanning stops, when you touch any button
         Stop Timer1 : Timer1 = 0
         Bitwait Bt1 , Set : Bitwait Bt3 , Set : Bitwait Bt2 , Set       'wait for releasing any button
         Start Timer3                                       'start timer3 for periodical reading RDS register
         Return
      End If
      If S.0 > 0 And T.7 > 0 And Kmitocet <> 960 Then       'automatic stop, when strong radio station appears
                                                            'exclude frequency 96.0MHz which is harmonics of 8MHz (processor frequency)
                                                            'variables S and T are read in "Read_rds" function. ( Rds(3) and Rds(4) )
         Stop Timer1 : Timer1 = 0
         Enable Ovf1 : Start Timer1
         Start Timer3
         Return
      End If

   Loop
End Sub

'*******************************************************************************
'***** Change Mono x Stereo  ***************************************************
'*******************************************************************************
Sub Stereo()
   Timer3 = 50000
   Timer1 = 0
   Enable Ovf1
   Start Timer1
   Toggle Audio.5                                           'Swap bit 5 of variable Audio = write register 02-High of RDS5807m
   Nastav                                                   'jump to the calculations, setting and display the frequency
   Waitms 500
End Sub

'*******************************************************************************
'***** Turn On/Off Bass boost **************************************************
'*******************************************************************************
Sub Bass()
   Timer3 = 50000
   Timer1 = 0
   Enable Ovf1
   Start Timer1
   Toggle Audio.4                                           'Swap  bit 4 of variable Audio = write register 02-high of RDS5807m
   Nastav                                                   'jump to the calculations, setting and display the frequency
   Waitms 500
End Sub

'*******************************************************************************
'***** Manual tunning **********************************************************
'*******************************************************************************
Sub Ladeni()
   Timer3 = 50000
   Stop Timer3
   Timer1 = 0
   Enable Ovf1
   Start Timer1

   If Re_b = 0 Then                                         'when switch-B is 0 then tune 1 step up
      Kmitocet = Kmitocet + 1
      If Kmitocet > 1080 Then Kmitocet = 870                'at the end of the band continue from beginning
   Else
      Kmitocet = Kmitocet - 1                               'when switch-B is not 0 then tune 1 step down
      If Kmitocet < 870 Then Kmitocet = 1080                'at the bottom of the band continue from the top
   End If
   Nastav                                                   'jump to the calculations, setting and display the frequency
   Lcdat 1 , 33 , "FM RADIO"                                'delete name of the station and display universal "FM RADIO"
   Start Timer3                                             'start timer3 for reading RDS
End Sub

'*******************************************************************************
'***** Save all the values into Eeprom *****************************************
'*******************************************************************************
Sub Uloz()
      Timer3 = 50000
      E_kmitocet = Kmitocet                                 'frequency
      E_volume = Volume                                     'volume
      E_audio = Audio                                       'audio (Stereo, Bass-boost)
      Disable Ovf1
      Stop Timer1
      Timer1 = 0
End Sub

'*******************************************************************************
'***** Set the volume  *********************************************************
'*******************************************************************************
Sub Nastav_volume()
      Svetlo = &HAF                                         'Turn On display...
      Glcdcmd Svetlo                                        '...just in case, it was Off.
      Bitwait Bt1 , Set                                     'Wait to release of the button1
      Vol_mem = 1                                           'Set the flag, we are in Volume setting
      Stop Timer3
      Timer3 = 50000
      Waitms 200
      Timer1 = 0
      Enable Ovf1
      Start Timer1
  '********* Show the starting volume **********************
      Setfont _font12x16                                    'set font 12x16 px
      Zobraz_volume = Volume - 20 : Zobraz_volume = Zobraz_volume / 2       'number to show on display
      Lcdat 1 , 1 , "           "
      Lcdat 1 , 30 , "Vol"
      Lcdat 1 , 65 , ":"
      Lcdat 1 , 78 , Zobraz_volume                          'show the number of volume
  '******** waiting for rotary encoder or buttons **********
      Do
         Debounce Re_a , 0 , _volume , Sub                  'wait for rotary encoder (jump to actual setting volume)
         Debounce Bt2 , 0 , Bt2_vidle , Sub                 'wait for button2 (Stereo, Bass-boost)
         Debounce Bt3 , 0 , Zhasni , Sub                    'wait for button3 (turn Off/On display)
         If Re_sw = 0 Then                                  'when switchin rotary encoder axis =0 then return
            Return                                          'to main loop
         End If
         If Bt1 = 0 Or Timer1 > 65000 Then                  'If push Bt1 or time>cca 8 sec then return
            E_volume = Volume                               'save volume into Eeprom
            Waitms 200
            Lcdat 1 , 13 , "          "                     'clear the line on the display
            If Pam = 1 Then                                 'When volume from memory presets...
               Nastav_pamet_b                               '...then set the memory preset back.
               Return
            Else
               Lcdat 1 , 1 , "           "                  'Else, when volume from manual tuning...
               Lcdat 1 , 1 , "T:"                           '...then just write T:(uning) and return.
            End If

            Timer1 = 0
            Start Timer3                                    'start RDS timer
            Return
         End If
      Loop
End Sub

'***** Actual setting the volume ***********************************************
Sub _volume()
   Timer1 = 0

   If Re_b = 0 Then                                         'If rotary encoder switch-B is 0 then...
      Volume = Volume + 2                                   '...increase the volume.
      If Volume > 100 Then Volume = 100                     'stop the maximal value
   Else                                                     'Else (if RE switch-B is 1)...
      Volume = Volume - 2                                   '...decrease the volume.
      If Volume < 20 Then Volume = 20                       'stop the minimal value
   End If
   Pwm2b = Volume                                           'send the value to PWM output
   Zobraz_volume = Volume - 20 : Zobraz_volume = Zobraz_volume / 2       'create the number for display
   Lcdat 1 , 78 , "    "
   Lcdat 1 , 78 , Zobraz_volume                             'show the number of volume
End Sub

'*******************************************************************************
'***** Calculations, setting and display the frequency and Audio params. *******
'*******************************************************************************
Sub Nastav()
   Svetlo = &HAF                                            'Turn On display...
   Glcdcmd Svetlo                                           '...just in case, it was Off.
   Rds_text = ""                                            'delete RDS text
   I2cstart                                                 'start I2C communication
   I2cwbyte &B00100000                                      'adress RD5807m + write
   I2cwbyte Audio                                           'write 02H Audio = &B11XX_0010  (xx=Stereo, Bass)
   I2cwbyte &B00001101                                      'write 02L (not necessary to change)
   Pll = Kmitocet - 870                                     'calculation frequency for 03H
   Dato0_byte = Pll / 4                                     'calculation frequency for 03H
   Waitus 5                                                 'wait (just to be sure)
   I2cwbyte Dato0_byte                                      'write frequency to register 03H
   Dato1_byte = Dato0_byte * 4 : Dato1_byte = Pll - Dato1_byte       'calculation frequency for 03L
   Dato1_byte = Dato1_byte * 64 : Dato1_byte.4 = 1          'calculation frequency for 03L
   I2cwbyte Dato1_byte                                      'write to register 03L
   I2cwbyte &B00000011                                      'write to register 04H (not necessary to change)
   I2cwbyte &B00000000                                      'write to register 04L (not necessary to change)
   I2cwbyte &B10001000                                      'write to register 05H (not necessary to change)
   I2cwbyte &B10001100                                      'write to register 05L volume 1111=max, 0000=min (volume by register in rds5807m is fixed)
   I2cwbyte &B00000000                                      'write to register 06H (not necessary to change)
   I2cstop                                                  'stop I2C communication

   'Show frequency
   Kmitocet_str = Str(kmitocet)                             'format string for display
   Kmitocet_str = Format(kmitocet_str , "###.#")
   Setfont Digi_16x32                                       'set font 16x32 px
   Lcdat 5 , 1 , "     "                                    'delete previous value
   Lcdat 5 , 1 , Kmitocet_str                               'display new value

   'Show M-ono / S-tereo
   Setfont Font8x8                                          'set font 8x8 px
   If Audio.5 = 1 Then
      Lcdat 8 , 90 , "M"
   Else
      Lcdat 8 , 90 , "S"
   End If

   'Show B-ass Boost
   If Audio.4 = 1 Then
      Lcdat 8 , 109 , "B"
   Else
      Lcdat 8 , 109 , " "
   End If

   Setfont _font12x16                                       'set font 12x16 px
End Sub

'*******************************************************************************
'***** Read from RDS registers *************************************************
'*******************************************************************************
Dim R As Byte
Sub Read_rds()
   For R = 1 To 16
      I2cstart                                              'start I2C communication
      I2cwbyte &H22                                         'write value to set READ
      I2cwbyte &H0A                                         ' b  Regno = &B00001010
      I2cstart
      I2cwbyte &H23                                         'first address
      I2crbyte Rds(1) , Ack                                 'read all the 12 bytes...
      I2crbyte Rds(2) , Ack                                 '...into array Rds(12)
      I2crbyte Rds(3) , Ack
      I2crbyte Rds(4) , Ack
      I2crbyte Rds(5) , Ack
      I2crbyte Rds(6) , Ack
      I2crbyte Rds(7) , Ack
      I2crbyte Rds(8) , Ack
      I2crbyte Rds(9) , Ack
      I2crbyte Rds(10) , Ack
      I2crbyte Rds(11) , Ack
      I2crbyte Rds(12) , Nack
      I2cstop                                               'stop I2C communication
'*******************************************************************************
'***** Display RDS text ********************************************************
'*******************************************************************************
      Dim Rds_4 As String * 2                               'declaration of new variables for RDS text
      Dim Rds_3 As String * 2                               'RDS text (8 characters) consists of...
      Dim Rds_2 As String * 2                               '... 4 x 2 characters
      Dim Rds_1 As String * 2
      Dim Chyby As Byte                                     'Chyby = Errors (defected bytes)
      Dim Rds_text_old As String * 8                        'not used
      S = Rds(3)
      T = Rds(4)

      A = Rds(8) : A = A And &B0000_0011                    '"A" is a possition of each 2 characters in the RDS text
      B = Rds(7) : Shift B , Right , 4                      'calculations of some RDS errors...
      Chyby = Rds(4)                                        '... Sorry, I dont remeber it well.

      If B = 0 And Chyby = 128 Then                         'When B=0 then valid data are in RDS registers
                                                            'Under some error conditions
         If A = 0 Then                                      'The first 2 characters
            Rds_1 = Chr(rds(11)) + Chr(rds(12))
         Elseif A = 1 Then                                  'The second 2 charactest
            Rds_2 = Chr(rds(11)) + Chr(rds(12))
         Elseif A = 2 Then                                  'The third 2 characters
            Rds_3 = Chr(rds(11)) + Chr(rds(12))
         Elseif A = 3 Then                                  'The fourth 2 characters
            Rds_4 = Chr(rds(11)) + Chr(rds(12))
         End If
         Rds_text = Rds_1 + Rds_2 + Rds_3 + Rds_4           'Combine it into 8 character txt
      End If

      Dim Rds_text_2 As String * 8
      Dim Rds_text_3 As String * 8
      Setfont _font12x16

     '***** Show the RDS text only, when 3x the same ***************************
     '***** Doesnt work properly ***********************************************
      If Rds_text = Rds_text_2 Then
         If Rds_text_2 = Rds_text_3 Then
            Lcdat 1 , 33 , Rds_text
         Else
            Rds_text_3 = Rds_text_2
         End If
      Else
         Rds_text_2 = Rds_text
         Lcdat 1 , 33 , "FM RADIO"
      End If
   Next R
   Baterie                                                  'Jump to the measuring of the battery voltage
   Timer3 = 50000
End Sub

'*******************************************************************************
'***** Save the radio-station into preset memory********************************
'*******************************************************************************
 Sub Uloz_pamet()                                           'save memory
   Stop Timer1 : Stop Timer3

   Nejvyssi_pamet = E_nejvyssi_pamet                        'Read the highest occupied position from Eeprom
   If Nejvyssi_pamet > 100 Then Nejvyssi_pamet = 0          'When out of range (first turn on after upload code)...
                                                            '... set it to 0.
   Nejvyssi_pamet = Nejvyssi_pamet + 1                      'increase the position by 1

   If Nejvyssi_pamet > 20 Then Nejvyssi_pamet = 20          'Now, when position higher than 20...   (you can set more, if you like)
                                                            '...stay at this number.
   Cislo_pameti = Nejvyssi_pamet                            'Set number of memory position = the highest position
   Pamet_jmeno = Rds_text                                   'Memory_name = RDS_text
   Lcdat 1 , 1 , "  "
   Lcdat 1 , 1 , Cislo_pameti                               'display number
   Lcdat 1 , 33 , Pamet_jmeno , 1                           'display name in inversion (in negative)
   Bitwait Re_sw , Set                                      'wait until you release the button
   Waitms 200                                               'wait more 200ms

   '***************************************************************************
   '**** Now, you have 3 options **********************************************
   '***************************************************************************
   Do
      Debounce Re_a , 0 , Vyber_pamet , Sub                 'Either change the number of the position...
                                                            'Vyber_pamet = choose the memory
      If Re_sw = 0 Then Exit Do                             '... or save it immediately...
      If Bt3 = 0 Then                                       '... or release it without saving...
         Lcdat 1 , 1 , "T:"
         Lcdat 1 , 33 , "        "
         Bitwait Bt3 , Set
         Waitms 200
         Start Timer3
         Exit Sub                                           '...and return into tunning mode.
      End If
   Loop

   '*******************************
   '**** save it ******************
   '*******************************
   E_pamet(cislo_pameti) = Kmitocet                         'Save frequency to appropriate item in Eeprom array
   E_pamet_jmeno(cislo_pameti) = Pamet_jmeno                'save Name to appropriate item in Eeprom array
   Lcdat 1 , 1 , "           "
   Lcdat 1 , 1 , Cislo_pameti                               'Display memory number
   Lcdat 1 , 33 , Pamet_jmeno                               'Display saved name of radio station
   If Nejvyssi_pamet = Cislo_pameti Then                    'When memory number is the highest...
      E_nejvyssi_pamet = Cislo_pameti                       '... then save it as it is.
   Else
      Nejvyssi_pamet = Nejvyssi_pamet - 1                   '...otherwise increase it by 1
      E_nejvyssi_pamet = Nejvyssi_pamet
   End If
   E_posledni_pamet = Cislo_pameti                          'Save number of the last saved memory into Eeprom
   Lcdat 1 , 33 , " SAVED  "                                'Display "SAVED"
   Bitwait Re_sw , Set                                      'wait for release of the button
   Waitms 2000                                              'wait 2 seconds more
   Nastav_pamet                                             'jump to the calculations, setting and display the frequency                                           '
 End Sub

 '**** choose the memory position number ***************************************
 '*** You can choose only any lower (already occupied) position ****************
 Sub Vyber_pamet()
   If Re_b = 0 Then                                         'Rotary encoder turned to the right
      Cislo_pameti = Cislo_pameti + 1                       'increase the position
      If Cislo_pameti > Nejvyssi_pamet Then Cislo_pameti = 1       'cant be higher than highest
   Else
      Cislo_pameti = Cislo_pameti - 1                       'decrease the position
      If Cislo_pameti < 1 Then Cislo_pameti = Nejvyssi_pamet       'cant be lower than 1
   End If
   Lcdat 1 , 1 , "  "
   Lcdat 1 , 1 , Cislo_pameti                               'display the memory position number
 End Sub
 '******************************************************************************
 '***** Use (listen) the radio station from memory *****************************
 '******************************************************************************
 Sub Nastav_pamet()
   Stop Timer1 : Stop Timer3                                'stop both 16bit timers
   Pam = 1                                                  'set the memory flag (Pam)
   E_pam = Pam                                              'save the flag memory into Eeprom
   Nejvyssi_pamet = E_nejvyssi_pamet                        'read the highest memory item from eeprom
   Cislo_pameti = E_posledni_pamet                          'read the number of the memory item in use from Eeprom
   If Cislo_pameti < 1 Or Cislo_pameti > 20 Then            'If the number of memory item is out of range...
                                                            ' (when first turned On after upload code)
      Lcdat 1 , 33 , "M: Empty" : Wait 3                    'display "it is empty"
      Lcdat 1 , 33 , "        "
      Start Timer3
      Pam = 0 : E_pam = Pam                                 'reset the  memory flag and...
      Return                                                '... return to tuning mode.
   End If
   Nastav_pamet_b                                           'Otherwise jump to set memory part B

   Bitwait Re_sw , Set                                      'wait until you release the button
   Waitms 200                                               'wait 200ms more
   Start Timer3                                             'start timer to measure time for cyclic battery measurement
   Do                                                       'Waiting for buttons in the memory items mode
      Debounce Bt1 , 0 , Nastav_volume , Sub                'Volume button
      Debounce Bt2 , 0 , Bt2_vidle , Sub                    'Button2 - fork
      Debounce Bt3 , 0 , Zhasni , Sub                       'Button3 - turn off the light
      Debounce Re_a , 0 , Pamet_vyber , Sub                 'turning rotary encoder
      If Re_sw = 0 Then                                     'Switch in rottary encoder axis
         Waitms 200
         Lcdat 1 , 1 , "           "                        'Clear the number of memory item
         Lcdat 1 , 1 , "T:"                                 'Display "T:" as signe of Tunning mode
         Start Timer3 : Timer1 = 0                          'start both timers
         Pam = 0 : E_pam = Pam                              'reset memory flag and save it into Eeprom
         Bitwait Re_sw , Set                                'wait until release the switch
         Return                                             'and return in manual Tuning mode
      End If
      If Timer3 > 30000 Then                                'When timer3 reaches this number...
         Timer3 = 0                                         '... reset it and...
         Baterie                                            '...jump to battery (voltage) measurement.
      End If

   Loop

 End Sub

Sub Pamet_vyber()                                           'Choose the memory item. Jump here from Debounce (above)
   Timer3 = 0
   If Re_b = 0 Then                                         'When rotary encoder turned to right...
      Cislo_pameti = Cislo_pameti + 1                       '... increase number of memory item
      If Cislo_pameti > Nejvyssi_pamet Then Cislo_pameti = 1       'If the number is higher then maximum, start from 1
   Else                                                     'When rotary encoder turned to left...
      Cislo_pameti = Cislo_pameti - 1                       '... decrease the number of memory item.
      If Cislo_pameti < 1 Then Cislo_pameti = Nejvyssi_pamet       'If the number is 0, start from highest.
   End If
   Nastav_pamet_b                                           'jump to set memory part B
   E_posledni_pamet = Cislo_pameti                          'Save into Eeprom the actual number of memory item
End Sub

Sub Nastav_pamet_b()                                        'set the memory item part B
   Kmitocet = E_pamet(cislo_pameti)                         'frequency = Eeprom memory(number)
      If Vol_mem = 0 Then
         Nastav
      Else
         Vol_mem = 0
      End If
   Lcdat 1 , 1 , "           "                              'clear the position with the number (or T:)
   Lcdat 1 , 1 , Cislo_pameti                               'display number
   Rds_text = E_pamet_jmeno(cislo_pameti)                   'RDS is now stopped and replaced with saved name
   Lcdat 1 , 33 , Rds_text                                  'display the name
End Sub

'*******************************************************************************
'*** Measuring of battery voltage **********************************************
'*******************************************************************************
Sub Baterie()
   Start Adc
   Volty = Getadc(3)                                        'read the ADC
   Stop Adc
   If Volty < V_limit Then                                  'If voltage is lover than limit...
      If Pam = 1 Then                                       'this condition is probably not necessary
                                                            'probably remainder from older version
         Lcdat 1 , 33 , "        "
      Else
         Lcdat 1 , 33 , "        "
      End If
      Waitms 300
      Lcdat 1 , 33 , "BAT LOW "                             '... Display warning.
   End If
End Sub
'*******************************************************************************
'***** functions included from external files **********************************
'*******************************************************************************
$include "clear_screen.bas"                                 'clear screen (display)
$include "Init_SSH1106.bas"                                 'initialization of SSH1106 display

'*******************************************************************************
'**** Fonts for display ********************************************************
'*******************************************************************************
''$include "font12x16.font"
$include "_font12x16.font"                                  'unproportional font 12x16 px
$include "font8x8.font"                                     'proportional font 8x8 px
''$include "font16x16.font"
$include "digi_16x32.font"                                  'special font 16x32 px (7 segment like)

'*******************************************************************************
'***** Title picture (not used) ************************************************
'*******************************************************************************
''Logo:
''   $bgf "ok1hdu.bgf"